// declare variables
variable "do_token" {
  type        = string
  description = "DigitalOcean Personal Access Token (with write access)"
}
variable "my_ip" {
  type        = string
  description = "IP range to open SSH to"
  default     = "0.0.0.0/0"
}
variable "ssh_fingerprint" {
  type        = string
  description = "ID of SSH key to use from DigitalOcean control panel"
}
variable "ssh_priv_key_path" {
  type        = string
  description = "Path to private SSH key"
  default     = "./modules/express_site/id_rsa"
}

// setup DigitalOcean provider
provider "digitalocean" {
  token = var.do_token
}

// Virtual Private Cloud for containing droplets
resource "digitalocean_vpc" "vpc" {
  name    = "LUG-demo-vpc"
  region  = "nyc1"
}

// Create the droplet (DigitalOcean speak for VPS)
resource "digitalocean_droplet" "express_server" {
  image     = "ubuntu-20-04-x64"  
  name      = "LUG-demo-droplet"
  region    = "nyc1"
  size      = "s-1vcpu-1gb"
  // add SSH keys
  ssh_keys  = [var.ssh_fingerprint]
  // add it to the previously created VPC
  vpc_uuid  = digitalocean_vpc.vpc.id

  // copy project in over SSH
  provisioner "file" {
    source      = "./modules/express_site/project"
    destination = "/root"

    connection {
      type = "ssh"
      user = "root"
      // grab the IP address of the droplet after it's created
      host = self.ipv4_address
      private_key = file("./modules/express_site/id_rsa")
    }
  }

  // setup the droplet over SSH
  provisioner "remote-exec" {
    inline = [
      "apt update",
      "apt install -y nodejs",
      "apt install -y npm",
      "cd /root/project",
      "npm install"
    ]
     
    connection {
      type = "ssh"
      user = "root"
      host = self.ipv4_address
      private_key = file("./modules/express_site/id_rsa")
    }
  }
}

// setup a firewall
resource "digitalocean_firewall" "firewall" {
  name = "LUG-demo-firewall"
  droplet_ids = [digitalocean_droplet.express_server.id]

  // SSH
  inbound_rule {
    protocol          = "tcp"
    port_range        = "22"
    source_addresses  = [var.my_ip]
  }

  // webapp
  inbound_rule {
    protocol          = "tcp"
    port_range        = "3000"
    source_addresses  = ["0.0.0.0/0", "::/0"]
  }

  // allow droplets to initiate with anyone
  outbound_rule {
    protocol              = "tcp"
    port_range            = "1-65535"
    destination_addresses = ["0.0.0.0/0", "::/0"]
  }
}

// provide droplet ID as an output
output "droplet_id" {
  description = "ID of Droplet"
  value       = digitalocean_droplet.express_server.id
}

